@@ -1,5 +1,5 @@
 from ._colors import get_srgb1
-from ._main import plot, plot_abs, plot_arg, plot_contours, plot_phase
+from ._main import Plotter, plot, plot_abs, plot_arg, plot_contours, plot_phase
 from ._tri import tricontour_abs, tripcolor
 from .benchmark import show_kovesi_test_image, show_test_function
 from .create import create_colormap, find_max_srgb_radius, show_circular, show_linear
@@ -20,4 +20,6 @@ __all__ = [
     #
     "tripcolor",
     "tricontour_abs",
+    #
+    "Plotter",
 ]
